@@ -7,16 +7,15 @@ import numpy as np
 import time
 import itertools
 import sys
-# from tqdm import tqdm
 from tqdm.auto import tqdm
 
-from aaindex.aaindex import aaindex
+from aaindex import aaindex1
 from .model import Model
 from .pyDSP import PyDSP
 from .evaluate import Evaluate
 from .pySAR import PySAR
 from .utils import *
-from .descriptors_ import Descriptors
+from .descriptors import Descriptors
 
 class Encoding(PySAR):
     """
@@ -36,20 +35,21 @@ class Encoding(PySAR):
     and its methods differ from the PySAR class by allowing for the encoding using
     all available features. To date, there are 566 indices supported in the AAI and
     pySAR supports 15 different descriptors. The features can be encoeed using
-    different combinations, for example, 1 2 or 3 descriptors can be used for the
+    different combinations, for example, 1, 2 or 3 descriptors can be used for the
     descriptor and AAI + Descriptor encoding strategies. In total, this class
     supports over 410,000 possible ways at which to numerically encode the
     protein sequences in the building of a predictive ML model for mapping these
     sequences to a particular activity, known as a Sequence-Activity-Relationship (SAR).
 
-    Attributes
+    Parameters
     ----------
-        :config_file (str): 
-            path to configuration file.
+    :config_file (str): 
+        path to configuration file with all required parameters for the pySAR encoding
+        pipeline.
 
     Methods
     -------
-    aai_encoding(self, aai_list=None, cutoff_index=1):
+    aai_encoding(aai_list=None, cutoff_index=1):
         encoding protein sequences using indices from the AAI.
     descriptor_encoding(desc_combo=1):
         encoding protein sequences using protein descriptors from descriptors module.
@@ -59,8 +59,8 @@ class Encoding(PySAR):
     """
     def __init__(self, config_file=""):
 
-        self.config_file = conf= config_file
-
+        self.config_file = config_file
+        #pass config file into parent pySAR class
         super().__init__(self.config_file)
 
         #create output directory to store all of the program's outputs
@@ -82,9 +82,10 @@ class Encoding(PySAR):
         ----------
         :aai_list : list (default=None)
             list of aai indices to use for encoding the predictive models, by default
-            all AAI indices will be used.
+            ALL AAI indices will be used.
         :cutoff_index : int (default=1)
-            set the proportion of iterations of AAI indices to actually encode -> mainly used for testing.
+            set the proportion of iterations of AAI indices to actually encode. This is mainly used for 
+            testing and the default of 1 means that all indices will be used as standard.
         :sort_by : str (default=R2)
             sort output dataframe by specified column/metric value, results sorted by R2 score by default.
 
@@ -92,7 +93,9 @@ class Encoding(PySAR):
         -------
         :aaindex_metrics_df : pd.DataFrame
             dataframe of calculated metric values from generated predictive models
-            encoded using indices in the AAI for the AAI encoding strategy.
+            encoded using indices in the AAI for the AAI encoding strategy. Output will 
+            be of the shape 566 x 8, where 566 is the number of indices that can be used
+            for the encoding and 8 is the results/metric columns.
         """
         #setting DSP params to None if not using them
         if not (self.use_dsp):
@@ -114,11 +117,11 @@ class Encoding(PySAR):
         rpd_ = []
         mae_ = []
         explained_var_ = []
-        index_count = 1        #counters to keep track of current index
+        # index_count = 1        #counters to keep track of current index
 
         #if no indices passed into aai_list then use all indices by default
-        if aai_list == None or aai_list == [] or aai_list == "":
-            all_indices = aaindex.record_codes()
+        if (aai_list == None or aai_list == [] or aai_list == ""):
+            all_indices = aaindex1.record_codes()
         elif isinstance(aai_list, str):   #if single descriptor input, cast to list
             all_indices = [aai_list]
         else:
@@ -146,11 +149,13 @@ class Encoding(PySAR):
         5.) Repeat steps 1 - 4 for all indices.
         6.) Output results into a final dataframe, save it and return.
         '''
+
         start = time.time() #start time counter
 
-        #using tqdm package to create a progress bar showing encoding progress
+        #using tqdm package to create a progress bar showing encoding progress,
         #   file=sys.stdout to stop error where iterations were printing out of order
-        for index in tqdm(all_indices[:cutoff_index], unit=" indices", position=0, desc="AAI Indices", file=sys.stdout):
+        for index in tqdm(all_indices[:cutoff_index], unit=" indices", position=0, 
+            desc="AAI Indices", file=sys.stdout):
 
             #get AAI indices encoding for sequences according to index var
             encoded_seqs = self.get_aai_encoding(index)
@@ -176,11 +181,11 @@ class Encoding(PySAR):
             Y_pred = self.model.predict()
 
             #initilaise instance of Evaluate class
-            eval = Evaluate(Y_test,Y_pred)
+            eval = Evaluate(Y_test, Y_pred)
 
             #append values/results from current AAI encoding iteration to lists
             index_.append(index)
-            category_.append(aaindex.get_category_from_record(index))
+            category_.append(aaindex1[index].category)
             r2_.append(eval.r2)
             rmse_.append(eval.rmse)
             mse_.append(eval.mse)
@@ -188,10 +193,12 @@ class Encoding(PySAR):
             mae_.append(eval.mae)
             explained_var_.append(eval.explained_var)
 
-        end = time.time()       #stop time counter
+        #stop time counter, work out elapsed time
+        end = time.time()      
         elapsed = end - start
+
         print('\n\n##############################################################')
-        print('Elapsed Time for AAI Encoding: {0:.3f} seconds'.format(elapsed))
+        print('Elapsed Time for AAI Encoding: {0:.3f} seconds.'.format(elapsed))
 
         #set columns in the output dataframe to each of the values/metrics lists
         aaindex_metrics_= aaindex_metrics_df.copy()
@@ -208,7 +215,7 @@ class Encoding(PySAR):
         if (sort_by not in aaindex_metrics_df.columns):
             sort_by = 'R2'
 
-        #sort results by R2
+        #sort results according to sort_by parameter (R2 by default)
         aaindex_metrics_ = aaindex_metrics_.sort_values(by=[sort_by], ascending=False)
 
         #save results dataframe, saved to OUTPUT_DIR by default.
@@ -219,16 +226,17 @@ class Encoding(PySAR):
     def descriptor_encoding(self, desc_list=None, desc_combo=1, cutoff_index=1, sort_by='R2'):
         """
         Encoding all protein sequences using the available physiochemical
-        and structural descriptors. The sequences can be encoded using combinations
-        of 1, 2 or 3 of these descriptors, dictated by the desc_combo input parameter:
-        set this to 1, 2 or 3 for what encoding combination to use, default is 1. Each
-        descriptor encoding will be used as the feature data to build the predictive
-        regression models. With 15 descriptors supported by pySAR this means there
-        can be 15, 105 and 455 total predictive models built for 1, 2 or 3 descriptors,
-        respecitvely. These totals may vary depending on the meta-parameters of each of 
-        the descriptors. The metrics evaluated from the model for each descriptor encoding
-        combination will be collated into a dataframe and saved and returned, with the 
-        results sorted by the R2 score, this can be changed using the sort_by parameter.
+        and structural descriptors from the custom-built protpy package. The sequences 
+        can be encoded using combinations of 1, 2 or 3 of these descriptors, dictated 
+        by the desc_combo input parameter: set this to 1, 2 or 3 for what encoding 
+        combination to use, default is 1. Each descriptor encoding will be used as the 
+        feature data to build the predictive regression models. With 15 descriptors 
+        supported by pySAR & protpy this means there can be 15, 105 and 455 total 
+        predictive models built for 1, 2 or 3 descriptors, respecitvely. These totals 
+        may vary depending on the meta-parameters on some of the descriptors. The 
+        metrics evaluated from the model for each descriptor encoding combination will 
+        be collated into a dataframe and saved and returned, with the results sorted 
+        by the R2 score, this can be changed using the sort_by parameter.
 
         Parameters
         ----------
@@ -238,15 +246,20 @@ class Encoding(PySAR):
             list of descriptors to use for encoding, by default all available descriptors
             in the descriptors module will be used for the encoding.
         :cutoff_index : int (default=1)
-            set the proportion of iterations of descriptors to actually encode -> mainly used for testing.
+            set the proportion of iterations of descriptors to actually encode. This is 
+            mainly used for testing and the default of 1 means that all descriptors will 
+            be used as standard.
         :sort_by : str (default=R2)
-            sort output dataframe by specified column/metric value, results sorted by R2 score by default.
+            sort output dataframe by specified column/metric value, results sorted by R2 
+            score by default.
 
         Returns
         -------
         :desc_metrics_df_ : pd.DataFrame
             dataframe of calculated metric values from generated predictive models
             encoded using all or selected descriptors for the descriptors encoding strategy.
+            Output will be of the shape 15 x 8, where 15 is the number of descriptors that 
+            can be used for the encoding and 8 is the results/metric columns.
         """
         #create dataframe to store output results from models
         desc_metrics_df = pd.DataFrame(columns=['Descriptor', 'Group', 'R2', 'RMSE',
@@ -261,21 +274,22 @@ class Encoding(PySAR):
         rpd_ = []
         mae_ = []
         explained_var_ = []
-        msle_ = []
-        desc_count = 1  #counters to keep track of current index & descriptor
+        # desc_count = 1  #counters to keep track of current index & descriptor
+
+        #create instance of descriptors class
+        desc = Descriptors(self.config_file)
 
         #if no descriptors passed into desc_list then use all descriptors by default,
         #   get list of all descriptors according to desc_combo value
-        desc = Descriptors(self.config_file)
-        if desc_list == None or desc_list == [] or desc_list == "":
+        if (desc_list == None or desc_list == [] or desc_list == ""):
             # desc = Descriptors(self.config_file)
             all_descriptors = desc.all_descriptors_list(desc_combo)
-        elif isinstance(desc_list,str):     #if single descriptor input, cast to list
+        elif (isinstance(desc_list,str)):     #if single descriptor input, cast to list
             # desc = Descriptors(self.config_file)
             all_descriptors = [desc_list]
         else:
             # desc = Descriptors(self.config_file)
-            if desc_combo == 2:
+            if (desc_combo == 2):
                 all_descriptors = list(itertools.combinations(desc_list, 2))
             elif desc_combo == 3:
                 all_descriptors = list(itertools.combinations(desc_list, 3))
@@ -301,35 +315,40 @@ class Encoding(PySAR):
         5.) Repeat steps 1 - 4 for all descriptors.
         6.) Output results into a final dataframe, save it and return, sorting by sort_by parameter.
         '''
-        for descr in tqdm(all_descriptors[:cutoff_index], unit=" descriptor", position=0, desc="Descriptors", file=sys.stdout):
 
-            desc_ = pd.DataFrame()           #reset descriptor DF and list
+        for descr in tqdm(all_descriptors[:cutoff_index], unit=" descriptor", position=0, 
+            desc="Descriptors", file=sys.stdout):
+
+            #reset descriptor DF and list
+            desc_ = pd.DataFrame()           
             descriptor_list = []
-            desc_count+=1          #counter to keep track of current descriptor
+
+            #counter to keep track of current descriptor
+            # desc_count+=1          
 
             #if using 2 or 3 descriptors, append each descriptor & its category to list
-            if desc_combo == 2 or desc_combo == 3:
+            if (desc_combo == 2 or desc_combo == 3):
                 for de in descr:
                     descriptor_list.append(getattr(desc, de))
                     descriptor_group_.append(desc.descriptor_groups[de])
-                desc_ = pd.concat(descriptor_list,axis=1) #concatenate descriptors
+                desc_ = pd.concat(descriptor_list, axis=1) #concatenate descriptors
             else:
                 desc_ = desc.get_descriptor_encoding(descr)
                 descriptor_group_.append(desc.descriptor_groups[descr])
 
-            X = desc_   #set training data to desc_ dataframe
+            #set training data to desc_ dataframe
+            X = desc_   
 
             #get protein activity values
             Y  = self.get_activity()
             '''
-            If using the PlsRegression algorithm and there is only 1 feature (1-dimension)
-            in the feature data X (e.g SOCNum) then create a new PLSReg model with the n_components
+            Note: If using the PlsRegression algorithm and there is only 1 feature (1-dimension)
+            in the feature data X (e.g SOCN) then create a new PLSReg model with the n_components
             parameter set to 1 instead of the default 2 - this stops the error:
             'ValueError - Invalid Number of Components: 2'
-
-            Also, get train/test split, fit model and predict activity of test data.
             '''
-            if X.shape[1] == 1 and repr(self.model) == "PLSRegression":
+            #get train/test split, fit model and predict activity of test data.
+            if (X.shape[1] == 1 and repr(self.model) == "PLSRegression"):
               tmp_model = Model('plsreg', parameters={'n_components':1})
               X_train, X_test, Y_train, Y_test  = tmp_model.train_test_split(X, Y, test_size=self.test_split)
               model_fit = tmp_model.fit()
@@ -351,17 +370,19 @@ class Encoding(PySAR):
             mae_.append(eval.mae)
             explained_var_.append(eval.explained_var)
 
-        end = time.time()           #stop counter
+        #stop counter and calculate elapsed time
+        end = time.time()           
         elapsed = end - start
+
         print('\n\n##############################################################')
-        print('Elapsed Time for Descriptor Encoding: {0:.3f} seconds\n'.format(elapsed))
+        print('Elapsed Time for Descriptor Encoding: {0:.3f} seconds.\n'.format(elapsed))
 
         #if using combinations of 2 or 3 descriptors, group every 2 or 3 descriptor
         #   groups into one element in the descriptor group list
-        if desc_combo == 2:
+        if (desc_combo == 2):
           descriptor_group_= [ ','.join(x) for x in zip(descriptor_group_[0::2],
             descriptor_group_[1::2]) ]
-        elif desc_combo == 3:
+        elif (desc_combo == 3):
           descriptor_group_= [ ','.join(x) for x in zip(descriptor_group_[0::3],
             descriptor_group_[1::3],  descriptor_group_[2::3]) ]
 
@@ -380,23 +401,24 @@ class Encoding(PySAR):
         if (sort_by not in desc_metrics_df_.columns):
             sort_by = 'R2'
 
-        #sort results by R2
+        #sort results according to sort_by parameter (R2 by default)
         desc_metrics_df_ = desc_metrics_df_.sort_values(by=[sort_by], ascending=False)
 
         #set save path according to the descriptor combinations type
-        if desc_combo == 2:
+        if (desc_combo == 2):
             save_path = 'desc_combo2_results'
-        elif desc_combo == 3:
+        elif (desc_combo == 3):
             save_path = 'desc_combo3_results'
         else:
             save_path = 'desc_results'
 
         #save results dataframe to specified save_path
-        save_results(desc_metrics_df_,save_path)
+        save_results(desc_metrics_df_, save_path)
 
         return desc_metrics_df_
 
-    def aai_descriptor_encoding(self, aai_list=None, desc_list=None, desc_combo=1, cutoff_index=1, sort_by='R2', print_results=True):
+    def aai_descriptor_encoding(self, aai_list=None, desc_list=None, desc_combo=1, 
+        cutoff_index=1, sort_by='R2'):
         """
         Encoding all protein sequences using each of the indices in the AAI as well
         as the protein descriptors. The sequences can be encoded using 1 AAI + 1 Descriptor,
@@ -405,31 +427,45 @@ class Encoding(PySAR):
         The protein spectra of the AAI indices will be generated if the config param 
         use_dsp is true, along with the class attributes: spectrum, window, convolution and filter.
         Each encoding will be used as the feature data to build the predictive
-        regression models. To date, there are 566 indices and pySAR supports 15
+        regression models. To date, there are 566 indices and pySAR/protpy supports 15
         descriptors so the encoding process will generate 8490, ~59000 and ~257000
         models, when using 1, 2 or 3 descriptors + AAI indices, respectively. These values
-        may vary depending on the meta-parameters of each descriptor. The metrics evaluated 
+        may vary depending on the meta-parameters on some of the descriptors. The metrics evaluated 
         from the model for each AAI + Descriptor encoding combination will be collated into a 
         dataframe and saved and returned, sorted by R2 score by default.
 
         Parameters
-        ----------
-        :aai_list : list (default = None)
-            **refer to aai_encoding(...) docstring.
-        :desc_list : list (default = None)
-            **refer to desc_encoding(...) docstring.
-        :desc_combo : int (default = 1)
-            **refer to desc_encoding(...) docstring.
-        :cutoff_index : int (default = 1)
-            **refer to aai_encoding(...) docstring.
-        :sort_by : str (default = R2)
-            **refer to aai_encoding(...) docstring.
+        ----------        
+        :aai_list : list (default=None)
+            list of aai indices to use for encoding the predictive models, by default
+            ALL AAI indices will be used.
+        :decs_list : list (default=None)
+            list of descriptors to use for encoding, by default all available descriptors
+            in the descriptors module will be used for the encoding.
+        :desc_combo : int (default=1)
+            combination of descriptors to use.
+        :cutoff_index : int (default=1)
+            set the proportion of iterations of AAI indices + descriptors to actually encode. 
+            This is mainly used for testing and the default of 1 means that all AAI indices + 
+            descriptors will be used as standard.
+        :sort_by : str (default=R2)
+            sort output dataframe by specified column/metric value, results sorted by R2 
+            score by default.
+
+            dataframe of calculated metric values from generated predictive models
+            encoded using all or selected descriptors for the descriptors encoding strategy.
+            Output will be of the shape 15 x 8, where 15 is the number of descriptors that 
+            can be used for the encoding and 8 is the results/metric columns.
 
         Returns
         -------
         :aai_desc_metrics_df_ : pd.DataFrame
             dataframe of calculated metric values from generated predictive models
-            encoded using AAI indices + descriptors encoding strategy.
+            encoded using AAI indices + descriptors encoding strategy. The output will
+            be of shape (566 * 15) x 10, (566 * 105) x 10, or (566 * 455) x 10, 
+            depending on the desc_combo param which dictates the combinations of 
+            descriptors to use with the indices. 10 represents the results/metrics
+            columns of the dataframe.
         """
         #setting DSP params to None if not using them
         if not (self.use_dsp):
@@ -454,31 +490,33 @@ class Encoding(PySAR):
         mae_ = []
         explained_var_ = []
 
-        index_count = 1     #counters to keep track of current index & descriptor
+        # index_count = 1     #counters to keep track of current index & descriptor
         desc_count = 1
 
         #if no indices passed into aai_list then use all indices by default
-        if aai_list == None or aai_list == [] or aai_list == "":
-            all_indices = aaindex.record_codes()
-        elif isinstance(aai_list, str):   #if single descriptor input, cast to list
+        if (aai_list == None or aai_list == [] or aai_list == ""):
+            all_indices = aaindex1.record_codes()
+        elif (isinstance(aai_list, str)):   #if single descriptor input, cast to list
             all_indices = [aai_list]
         else:
             all_indices = aai_list
 
+        #create instance of Descriptors class
+        desc = Descriptors(self.config_file)
+
         #if no descriptors passed into desc_list then use all descriptors by default,
         #   get list of all descriptors according to desc_combo value
-        desc = Descriptors(self.config_file)
-        if desc_list == None or desc_list == [] or desc_list == "":
+        if (desc_list == None or desc_list == [] or desc_list == ""):
             # desc = Descriptors(self.config_file)
             all_descriptors = desc.all_descriptors_list(desc_combo)
-        elif isinstance(desc_list, str):     #if single descriptor input, cast to list
+        elif (isinstance(desc_list, str)):     #if single descriptor input, cast to list
             # desc = Descriptors(self.config_file)
             all_descriptors = [desc_list]
         else:
             # desc = Descriptors(self.config_file)
-            if desc_combo == 2:
+            if (desc_combo == 2):
                 all_descriptors = list(itertools.combinations(desc_list, 2))
-            elif desc_combo == 3:
+            elif (desc_combo == 3):
                 all_descriptors = list(itertools.combinations(desc_list, 3))
             else:
                 all_descriptors = desc_list
@@ -513,13 +551,14 @@ class Encoding(PySAR):
         6.) Repeat steps 1 - 5 for all indices in the AAI.
         7.) Output results into a final dataframe, save it and return, sort by sort_by parameter.
         '''
+        
         for index in tqdm(all_indices[:cutoff_index], unit=" indices", desc="AAI Indices"):
 
             #get AAI indices encoding for sequences according to index var
-            encoded_seqs = self.get_aai_encoding(index, print_results=True)
+            encoded_seqs = self.get_aai_encoding(index)
 
             #generate protein spectra from pyDSP class if use_dsp is true
-            if self.use_dsp:
+            if (self.use_dsp):
                 pyDSP = PyDSP(self.config_file, protein_seqs=encoded_seqs)
                 pyDSP.encode_seqs()
                 X_aai = pd.DataFrame(pyDSP.spectrum_encoding)
@@ -532,36 +571,37 @@ class Encoding(PySAR):
                 #reset descriptor DF and list
                 desc_ = pd.DataFrame()
                 descriptor_list = []
-                desc_count+=1   #counter to keep track of current descriptor
+
+                #counter to keep track of current descriptor
+                desc_count+=1   
 
                 #if using 2 or 3 descriptors, append each descriptor & its category to list
-                if desc_combo == 2 or desc_combo == 3:
+                if (desc_combo == 2 or desc_combo == 3):
                     for de in descr:
                         descriptor_list.append(getattr(desc, de)) #get descriptor attribute
-                        # descriptor_list.append(desc.get_descriptor_encoding(de))
                         descriptor_group_.append(desc.descriptor_groups[de])
                     desc_ = pd.concat(descriptor_list, axis=1) # check if this is needed here
 
                 #if only using 1 descriptor
                 else:
                     desc_ = getattr(desc, descr)    #get descriptor attribute
-                    # desc_ = desc.get_descriptor_encoding(descr)
                     descriptor_group_.append(desc.descriptor_groups[descr])
 
-                # X = desc_   #set training data to descriptor dataframe
+                #set training data to descriptor dataframe
+                # X = desc_   
                 X = pd.concat([desc_, X_aai], axis=1)
 
                 #get protein activity values
                 Y  = self.get_activity()
 
                 '''
-                If using the PlsRegression algorithm and there is only 1 feature (1-dimension)
+                Note: If using the PlsRegression algorithm and there is only 1 feature (1-dimension)
                 in the feature data X then create a new PLSReg model with the n_components
                 parameter set to 1 instead of the default 2 - this stops the error:
                 'ValueError - Invalid Number of Components: 2.'
-
-                Also, get train/test split, fit model and predict activity of test data.
                 '''
+
+                #get train/test split, fit model and predict activity of test data.
                 if X.shape[1] == 1 and repr(self.model) == "PLSRegression":
                   tmp_model = Model('plsreg',parameters={'n_components':1})
                   X_train, X_test, Y_train, Y_test  = tmp_model.train_test_split(X, Y)
@@ -577,7 +617,7 @@ class Encoding(PySAR):
 
                 #append values/results from current encoding iteration to lists
                 index_.append(index)
-                index_category_.append(aaindex.get_category_from_record(index))
+                index_category_.append(aaindex1[index]['category'])
                 descriptor_.append(descr)
                 r2_.append(eval.r2)
                 rmse_.append(eval.rmse)
@@ -588,17 +628,19 @@ class Encoding(PySAR):
 
             desc_count = 1  #reset descriptor counter
 
-        end = time.time()           #stop counter
+        #stop counter and calculate elapsed time
+        end = time.time()           
         elapsed = end - start
+
         print('\n##############################################################')
-        print('Elapsed Time for AAI + Descriptor Encoding: {0:.3f} seconds'.format(elapsed))
+        print('Elapsed Time for AAI + Descriptor Encoding: {0:.3f} seconds.'.format(elapsed))
 
         #if using combinations of 2 or 3 descriptors, group every 2 or 3 descriptor
         #   groups into one element in the descriptor group list
-        if desc_combo == 2:
+        if (desc_combo == 2):
           descriptor_group_= [ ','.join(x) for x in zip(descriptor_group_[0::2],
             descriptor_group_[1::2]) ]
-        elif desc_combo == 3:
+        elif (desc_combo == 3):
           descriptor_group_= [ ','.join(x) for x in zip(descriptor_group_[0::3],
             descriptor_group_[1::3],  descriptor_group_[2::3]) ]
 
@@ -619,14 +661,14 @@ class Encoding(PySAR):
         if (sort_by not in aai_desc_metrics_df_.columns):
             sort_by = 'R2'
 
-        #sort results by R2
+        #sort results according to sort_by parameter (R2 by default)
         aai_desc_metrics_df_ = aai_desc_metrics_df_.sort_values(by=[sort_by], ascending=False)
 
         #set save path according to the descriptor combinations type
-        if desc_combo == 2:
-            save_path = 'aaindex_descCombo2_results'
-        elif desc_combo == 3:
-            save_path = 'aaindex_descCombo3_results'
+        if (desc_combo == 2):
+            save_path = 'aaindex_desc_combo2_results'
+        elif (desc_combo == 3):
+            save_path = 'aaindex_desc_combo3_results'
         else:
             save_path = 'aaindex_desc_results'
 
@@ -639,8 +681,7 @@ class Encoding(PySAR):
 
     def __str__(self):
         return "Instance of Encoding Class with attribute values: Dataset: {}, Activity: {}, Algorithm: {}, Parameters: {}, Test Split: {}.".format(
-                    self.dataset, self.activity,self.algorithm, self.parameters, self.test_split
-        )
+                    self.dataset, self.activity,self.algorithm, self.parameters, self.test_split)
 
     def __repr__(self):
-        return "<{} >".format(self)
+        return "<{} >".format(self)
\ No newline at end of file
